import QtQml
import QtQuick
import QtQuick.Layouts
import QtQuick.Controls

Pane {
    id: root
    width:  540
    height: 420
    property color extra_bg_col

    ColumnLayout {
        anchors.fill: parent
        spacing: 16
        
        GroupBox {
            Layout.fillWidth: true
            Layout.fillHeight: true
            //Layout.minimumHeight: 140
            //Layout.maximumHeight: 140
            title: "Settings"
            
            GridLayout {
                id: settingsGridLayout
                anchors.fill: parent
                columns: 5

                Item { Layout.fillWidth: true } // horizontal spacer item
                Label {
                    text: "Selected volume map (VMP):"
                }
                SpinBox {
                    id: selectedVMPSpBx
                    Layout.maximumWidth: 60
                    from:  1
                    to:    1
                    value: 1
                    onValueChanged: root.getValuesOfCurrentVMP(value)
                    editable: false
                }
                TextField {
                    id: selectedVMPNameEd
                    placeholderText: "vmp name"
                    //Layout.fillWidth: true
                    Layout.minimumWidth: 100
                    //Layout.maximumWidth: 100
                }
                Item { Layout.fillWidth: true } // horizontal spacer item
                
                Item { Layout.fillWidth: true } // horizontal spacer item
                Label {
                    text: "Estimated smoothness of VMP (FWHM [voxel]):"
                }
                TextField {
                    id: smoothnessFWHMEd
                    placeholderText: ""
                    Layout.fillWidth: true
                    Layout.maximumWidth: 60
                }
                Button {
                    text: "Estimate"
                    onClicked: root.onEstimateMapSmoothness()
                }
                Item { Layout.fillWidth: true } // horizontal spacer item

                Item { Layout.fillWidth: true } // horizontal spacer item
                Label {
                    text: "Number of iterations (to try, use 10):"
                }
                SpinBox {
                    id: nrIterationsSpBx
                    Layout.maximumWidth: 60
                    from: 10
                    to: 100000
                    value: 1000
                    editable: true
                    // live: true // enable when using Qt >= 6.6
                }
                Item {}
                Item { Layout.fillWidth: true } // horizontal spacer item
                
                Item { Layout.fillWidth: true } // horizontal spacer item
                Label {
                    text: "Cluster-defining threshold (CDT) - p value:"
                }
                TextField {
                    id: initialPValueEd
                    placeholderText: "threshold"
                    Layout.fillWidth: true
                    Layout.maximumWidth: 60
                    ToolTip.text: "Uncorrected voxel-level threshold"
                }
                Button {
                    // implicitWidth: 40
                    text: "Check"
                    onClicked: root.onCheckInitialPValue()
                }
                Item { Layout.fillWidth: true } // horizontal spacer item
            }
        }
        
        Label {
            text: "Information"
        }
            
        ScrollView {
            id: messageScrollView
            Layout.fillWidth: true
            //Layout.fillHeight: true
            Layout.minimumHeight: 120
            // Layout.maximumHeight: 80
        
            TextArea {
                id: infoTA
                readOnly: true
                textFormat: TextEdit.MarkdownText
                background: Rectangle {
                    color: Qt.rgba(0.9, 0.1, 0.1, 1.0)
                }
                Component.onCompleted: if(bv.BVTheme) infoTA.background.color = root.extra_bg_col
            }
        }
    
        RowLayout {
            id: buttonsRow
            spacing: 8
            
            Item { Layout.fillWidth: true }
            
            Button {
                id: quitBtn
                text: "Quit"
                onClicked: bv.FinishPlugin() // required for plugin GUI frontend, use CloseDialog() for pure GUI scripts 
            }
            Button {
                id: addSubjectBtn
                text: "GO"
                onClicked: root.calculateClusterThreshold()
            }
        }
    }
    
    Timer {
        id: timerCheckVMPs
        interval: 1000; running: false; repeat: false
        onTriggered: root.checkForVMPs()
    }
        
    Component.onCompleted: {
        // bv.SetGUIScriptWindowTitle("Cluster Thresholding Plugin"); // set title of GUI Script window - if GUI is used as plugin frontend, title will be set from plugin code
        bv.SetMinimumSizeOfPluginWindow(500, 420);

        console.log("Plugin ready"); // + "  parent width x height: " + parent.width + " " + parent.height)
        //bv.PrintToLog("Plugin> ClusterThresh GUI plugin running...");
        setMessage("Plugin ready!");        

        // define colors for information text area background
        if(bv.BVTheme) {
            if(bv.DarkMode) root.extra_bg_col = Qt.rgba(0.16, 0.16, 0.16, 1.0)
            else            root.extra_bg_col = Qt.rgba(0.86, 0.86, 0.86, 1.0)
        }
        
        timerCheckVMPs.start();
    }
    
    function checkForVMPs()
    {        
        bv.SetPluginStringParameter("Command", "CheckForVMPs");
        var ok = bv.ExecutePlugin();
        if(ok) {
            var n_maps =  bv.GetPluginIntParameter("N_VMPs");
            var sel_map = bv.GetPluginIntParameter("FirstMapShown");
            selectedVMPSpBx.from  = 1;
            selectedVMPSpBx.to    = n_maps;
            selectedVMPSpBx.value = sel_map
            addMessage("VMPs available - current map: **" + sel_map + "**");
            
            // get vmp values of current (1st map)
            getValuesOfCurrentVMP(sel_map);
        }
        else {
            addMessage("**No VMPs available** - load or generate volume maps and try again!");
            var text = "No VMPs available!";
            var infoText = "Load or generate volume maps and try again!";
            bv.MessageBox(text, infoText);
            bv.FinishPlugin();
        }
    }
    
    function getValuesOfCurrentVMP(map_i)
    {
        bv.SetPluginStringParameter("Command", "GetVMPValues");
        bv.SetPluginIntParameter("VMP_Index", map_i); // here 1-based, in backend 0-based
        var ok = bv.ExecutePlugin();
        if(ok) {
            // get vmp name and put it in text field
            var VMPName = bv.GetPluginStringParameter("VMP_Name")
            selectedVMPNameEd.text = VMPName;
            // get vmp thresh and put in text field
            var VMPThreshP = bv.GetPluginFloatParameter("VMP_ThreshP")
            initialPValueEd.text = VMPThreshP.toFixed(5);
        }
    }

    function onCheckInitialPValue()
    {
        var cur_p = parseFloat(initialPValueEd.text);
        addMessage("Checking value of CDT p value: " + cur_p + ":");
        if(cur_p > 0.001) {
            //addMessage2("- Initial CDT p value '" + cur_p + "' is larger than **0.001** - *adjustment recommended*!");
            var text = "The intial threshold (p = " + cur_p + ") is larger than 0.001. Do you want to adjust the threshold?";
            var infoText = "To avoid potentially inflated false positive rates, it is strongly suggested to use a value of p = 0.001 or smaller (see Help for details).";
            var yes = bv.YesNoMessageBox(text, infoText);
            if(yes) {
                initialPValueEd.text = "0.001";
                addMessage2("- Initial CDT p value has been adjusted to **0.001**");
                cur_p = 0.001;
            }
        }
        else {
            addMessage2("- Initial CDT p value <= 0.001 - *no adjustment needed*!");
        }
        // we set map value in any case since user might have changed value
        bv.SetPluginStringParameter("Command", "SetInitialP");
        bv.SetPluginIntParameter("VMP_Index", selectedVMPSpBx.value); // set before, but just to be sure
        bv.SetPluginFloatParameter("VMP_InitialP", cur_p);
        bv.ExecutePlugin();
    }
    
    function onEstimateMapSmoothness()
    {
        bv.SetPluginStringParameter("Command", "EstimateMapSmooth");
        bv.SetPluginIntParameter("VMP_Index", selectedVMPSpBx.value); // set before, but just to be sure
        bv.ExecutePlugin();
        var VMPSmoothFWHM = bv.GetPluginFloatParameter("VMP_SmoothFWHM");
        smoothnessFWHMEd.text = VMPSmoothFWHM.toFixed(3);
    }
    
    function calculateClusterThreshold()
    {
        if(smoothnessFWHMEd.text == "") {
            var text = "Map smoothness not set!";
            var infoText = "Click on 'Estimate' and re-launch cluster-level threshold calculation!";
            bv.MessageBox(text, infoText);
            return;
        }
        bv.SetPluginStringParameter("Command", "CalculateClusterThresh");
        bv.SetPluginIntParameter("VMP_Index", selectedVMPSpBx.value); // set before, but just to be sure
        bv.SetPluginFloatParameter("VMP_SmoothFWHM", parseFloat(smoothnessFWHMEd.text));
        bv.SetPluginFloatParameter("VMP_InitialP", parseFloat(initialPValueEd.text));
        bv.SetPluginIntParameter("NrOfIterations", nrIterationsSpBx.value); // set before, but just to be sure

        var success = bv.ExecutePlugin();
        if(success) {
            var corrected_thresh = bv.GetPluginIntParameter("CLUSTER_THRESH");
            addMessage("Cluster-level threshold calculated successfully!")
            addMessage2("- Applied cluster-level threshold (p=0.05) = **" + corrected_thresh + "**");
        }
        else
            addMessage("**ERROR** - cluster-level threshold  could not be determined!")
    }

    function addMessage(text)
    {
        infoTA.insert(infoTA.length, "\n" + text);
        infoTA.cursorPosition = infoTA.length
    }
    function addMessage2(text)
    {
        infoTA.insert(infoTA.length, text);
        infoTA.cursorPosition = infoTA.length
    }    
    function setMessage(text)
    {
        infoTA.text = text
    }
}
