import QtQml
import QtQuick
import QtQuick.Layouts
import QtQuick.Controls

Pane {
    id: root
    width: 400
    height: 250

    ColumnLayout {
        id: mainLayout
        anchors.fill: parent
        anchors.margins: 11
        spacing: 16

        GroupBox {
            id: rangeBox
            title: "Intensity range"
            Layout.fillWidth: true

            RowLayout {
                id: rangeRowLayout
                anchors.fill: parent

                Item { Layout.fillWidth: true } // horizontal spacer item
                Label {
                    text: "From:"
                }
                SpinBox {
                    id: rangeFrom
                    Layout.maximumWidth: 60
                    from: 0
                    to: 255
                    value: 0
                    editable: true
                }
                Item { Layout.fillWidth: true } // horizontal spacer item
                Label {
                    text: "To:"
                }
                SpinBox {
                    id: rangeTo
                    Layout.maximumWidth: 60
                    from: 0
                    to: 255
                    value: 255
                    editable: true
                }
                Item { Layout.fillWidth: true } // horizontal spacer item
            }
        }

        GroupBox {
            id: voxelsBox
            title: "Voxels in intensity range"
            Layout.fillWidth: true

            RowLayout {
                id: voxelsRowLayout
                anchors.fill: parent

                Label {
                    text: "No. of voxels:"
                }
                TextField {
                    id: nvoxelsText
                    placeholderText: "# voxels"
                    Layout.fillWidth: true
                }
                Button {
                    text: "Count"
                    onClicked: nvoxelsText.text = root.onCountVoxels(rangeFrom.value, rangeTo.value)
                }
            }
        }

        Item {  // spacer item
            Layout.fillHeight: true
        }

        RowLayout {
            id: buttonsRow

            Button {
                id: backupBtn
                text: "Backup"
                onClicked: root.onBackup()
            }
            Button {
                id: undoBtn
                text: "Undo"
                enabled: false
                onClicked: root.onUndo()
            }
            Item { Layout.fillWidth: true }
            Button {
                id: quitBtn
                text: "Quit"
                onClicked: BrainVoyager.CloseDialog()
            }
            Button {
                id: invertBtn
                text: "Invert"
                onClicked: root.onInvert()
            }
        }
    }        // end of ColumnLayout

    Component.onCompleted: {
        console.log("GUI Script ready") // + "  parent width x height: " + parent.width + " " + parent.height)
        BrainVoyager.PrintToLog("Script GUI> Hello from QML GUI Script")
        BrainVoyager.SetMinimumSizeOfGUIScriptWindow(400, 250);
        // BrainVoyager.SetMaximumSizeOfGUIScriptWindow(400, 250);
    }

    // In the "ExampleGUIPlugin" version, this function calls into C/C++ plugin code to massively speed up processing
    function onCountVoxels(irange_from, irange_to)
    {
        BrainVoyager.PrintToLog("Script GUI> Dialog's Count Voxels button clicked");

        var n_voxels = 0;
        var docVMR = BrainVoyager.ActiveDocument
        let dim_x = docVMR.DimX;
        let dim_y = docVMR.DimY;
        let dim_z = docVMR.DimZ;
        for(var x = 0; x<dim_x; x++) {
            //BrainVoyager.PrintToLog((x+1) + " / " + dim_x) // show progress to user in Log
            for(var y = 0; y<dim_y; y++) {
                for(var z = 0; z<dim_z; z++) {
                    var val = docVMR.GetVoxelIntensity(x, y, z)
                    if((val >= irange_from) && (val <= irange_to))
                        n_voxels += 1;
                }
            }
        }
        
        return n_voxels;
    }

    function onInvert()
    {
        BrainVoyager.PrintToLog("Script GUI> Dialog's Invert button clicked");

        let irange_from = rangeFrom.value
        let irange_to   = rangeTo.value
        
        var docVMR = BrainVoyager.ActiveDocument
        let dim_x = docVMR.DimX;
        let dim_y = docVMR.DimY;
        let dim_z = docVMR.DimZ;
        for(var x = 0; x<dim_x; x++) {
            //BrainVoyager.PrintToLog((x+1) + " / " + dim_x) // show progress to user in Log
            for(var y = 0; y<dim_y; y++) {
                for(var z = 0; z<dim_z; z++) {
                    var val = docVMR.GetVoxelIntensity(x, y, z)
                    if((val >= irange_from) && (val <= irange_to))
                        docVMR.SetVoxelIntensity(x, y, z, 225 - val)
                }
            }
        }
        // update VMR View:
        docVMR.UpdateView()
    }

    property var backup_vmr: []; // global array variable for backup of VMR intensities

    function onBackup()
    {
        BrainVoyager.PrintToLog("Script GUI> Dialog's Backup button clicked - storing VMR intensities for undo");

        // save intensities in backup-array
        var docVMR = BrainVoyager.ActiveDocument
        let dim_x = docVMR.DimX;
        let dim_y = docVMR.DimY;
        let dim_z = docVMR.DimZ;
        let dim_xy = dim_x * dim_y;
        for(var x = 0; x<dim_x; x++) {
            //BrainVoyager.PrintToLog((x+1) + " / " + dim_x) // show progress to user in Log
            for(var y = 0; y<dim_y; y++) {
                for(var z = 0; z<dim_z; z++) {
                    backup_vmr[z*dim_xy + y*dim_x + x] = docVMR.GetVoxelIntensity(x, y, z);
                }
            }
        }
        // enable 'Undo' button
        undoBtn.enabled = true
    }
    function onUndo()
    {
        BrainVoyager.PrintToLog("Script GUI> Dialog's Undo button clicked - restoring original VMR intensities");

        // restore original intensities from backup-array
        var docVMR = BrainVoyager.ActiveDocument
        let dim_x = docVMR.DimX;
        let dim_y = docVMR.DimY;
        let dim_z = docVMR.DimZ;
        let dim_xy = dim_x * dim_y;
        for(var x = 0; x<dim_x; x++) {
            //BrainVoyager.PrintToLog((x+1) + " / " + dim_x) // show progress to user in Log
            for(var y = 0; y<dim_y; y++) {
                for(var z = 0; z<dim_z; z++) {
                    docVMR.SetVoxelIntensity(x, y, z, backup_vmr[z*dim_xy + y*dim_x + x]);
                }
            }
        }
        // update VMR View:
        docVMR.UpdateView()
    }
}
