import QtQml
import QtQuick
import QtQuick.Layouts
import QtQuick.Controls
import QtQuick.Dialogs

Pane {
    id: root
    width: 620
    height: 400
    padding: 10
    property color extra_bg_col
    property color extra_hl_col
    
    // we define a column layout with a tab bar on top for switching between two pages below
    // the two pages are defined in separate files "RFXGCM_Page1.qml" and "RFXGCM_Page2.qml" to avoid clutter
    // finally we add a bottom row with buttons to quit and run the plugin's main function.
    ColumnLayout {
        anchors.fill: parent
        spacing: 0
    
        Item {
            height: 8 // just a bit of extra margin at top
        }    
        
        TabBar {
            id: tabbar
            currentIndex: 0
            Layout.fillWidth: true
            
            TabButton {
                text: " Input Data "
                //width: implicitWidth
            }
            TabButton {
                text: " Epoch Selection "
                //width: implicitWidth
            }
        }
        
        Item {  // spacer item
            height: 6
        }
        
        StackLayout {
            id: stackly
            Layout.fillWidth: true  // width: 620
            Layout.fillHeight: true // height: 350
            currentIndex: tabbar.currentIndex
        
            // 1st page
            RFXGCM_Page1 {
                id: pageInputData
            }

            // 2nd page
            RFXGCM_Page2 {
                id: pageEpochSelection
            }
        }
        
        Item {  // spacer item
            height: 10
        }

        RowLayout {
            id: buttonsRow
            spacing: 8
            
            Item { Layout.fillWidth: true }
            Button {
                id: quitBtn
                Layout.bottomMargin: 8
                text: "Quit"
                onClicked: bv.FinishPlugin() // required for plugin GUI frontend, use CloseDialog() for pure GUI scripts 
            }
            Button {
                id: invertBtn
                Layout.rightMargin: 6
                Layout.bottomMargin: 8
                text: "GO"
                onClicked: root.onGo()
            }
        }
    }

    // Here we define some data models (essentially lists) that are filled with dummy values (entries can be removed)
    ListModel {
        id: voisModel

        ListElement {
            idx: 1
            name: "VOI 1 - very very very long"
        }
        ListElement {
            idx: 2
            name: "VOI 2"
        }
    }
    ListModel {
        id: vtcsModel
        ListElement {
            idx: 1
            name: "VTC 1"
            path: "/dummy/path"
        }
        ListElement {
            idx: 2
            name: "VTC 2"
            path: "/dummy/path"
        }
    }
    ListModel {
        id: condsModel

        ListElement {
            idx: 1
            name: "Condition 1"
        }
        ListElement {
            idx: 2
            name: "Condition 2"
        }
    }
    
    Component.onCompleted: {
        bv.SetGUIScriptWindowTitle("RFX-GCM GUI Plugin") // set title of GUI script window - if this is used for plugin frontend, the title will be set cia C/C++ plugin code

        console.log("Plugin ready") // + "  parent width x height: " + parent.width + " " + parent.height)
        bv.PrintToLog("Plugin GUI> RFX-GCM GUI plugin running...")
        // for final version: clear models in case they contain dummy data
        voisModel.clear()
        vtcsModel.clear()
        condsModel.clear()
        
        // define colors for list view background and highlight
        if(bv.BVTheme) {
            if(bv.DarkMode) {
                root.extra_bg_col = Qt.rgba(0.08, 0.08, 0.08, 1.0)
                root.extra_hl_col = Qt.rgba(0.15, 0.35, 0.6, 1.0)
            }
            else {
                root.extra_bg_col = Qt.rgba(0.86, 0.86, 0.86, 1.0)
                root.extra_hl_col = Qt.rgba(0.3, 0.7, 1.0,  1.0)
            }
        }
        else { // OS theme
            if(bv.DarkMode) root.extra_hl_col = Qt.rgba(0.15, 0.35, 0.6, 1.0)
            else            root.extra_hl_col = Qt.rgba(0.3, 0.6, 1.0,  1.0)
        }

        var docVMR = bv.ActiveDocument;
        if(docVMR === "null") {
            bv.PrintToLog("No document available!")
            bv.FinishPlugin()
        }
        if(docVMR.IsVMR() === false) {
            bv.PrintToLog("Current document is not a VMR file!")
            bv.FinishPlugin()
        }

        bv.SetPluginStringParameter("COMMAND", "StartCheck");
        var VOIFile = docVMR.VOIFileName;
        bv.SetPluginStringParameter("VOIFile", VOIFile); // if there is a VOI file, load and show VOI names in ListWidget
        if( !bv.ExecutePlugin() ) // prereqs not met
            bv.FinishPlugin();

        root.fillVOIList();
    }
    
    // JavaScript functions to handle interactions with dialog and to send ommands and values to C++ plugin
    //

    function getNewVOIFile()
    {
        let filename = bv.BrowseFile("Select VOI File", "*.voi")
        console.log("selected voi file: " + filename)
        if(filename !== "") {
            pageInputData.voiFileNameTF.text = filename
            // let plugin load VOI file:
            bv.SetPluginStringParameter("VOIFile", filename);
            bv.SetPluginStringParameter("COMMAND", "StartCheck"); // use special command, e.g "LoadVOIFile"
            bv.ExecutePlugin();
            root.fillVOIList();
        }
        else {
            pageInputData.voiFileNameTF.text = ""
            voisModel.clear()
        }
    }
    function fillVOIList()
    {
        var NVOIs = bv.GetPluginIntParameter("NrOfVOIsAvailable");

        var docVMR = bv.ActiveDocument;

        pageInputData.voiFileNameTF.text = docVMR.VOIFileName;
        bv.PrintToLog("VOI File Name: " + docVMR.VOIFileName);
        voisModel.clear()

        for(var i=0; i<NVOIs; i++)
        {
            var VOINameID = "NameOfVOI-" + (i+1)
            var VOIName = bv.GetPluginStringParameter(VOINameID)
            voisModel.append({"idx": i+1, "name": VOIName})
        }
    }

    function onAddVTCFile() {
        var VTCFilePath = bv.BrowseFile("Select VTC File", "VTC files (*.vtc)");
        if(VTCFilePath == "")
            return;
        bv.PrintToLog("Script>  Selected VTC File: " + VTCFilePath);
        let N = vtcsModel.count
        let VTCFile = bv.GetFileFromFilePath(VTCFilePath)
        let VTCPath = bv.GetPathFromFilePath(VTCFilePath)
        vtcsModel.append({"idx": N+1, "name": VTCFile, "path": VTCPath})
    }

    function onLoadVTCsFromMDM()
    {
        var MDMFilePath = bv.BrowseFile("Select MDM File", "MDM files (*.mdm)");

        var VTCList = new Array();
        VTCList = bv.GetVTCsOfMDM(MDMFilePath);

        vtcsModel.clear();
        for(var i=0; i<VTCList.length; i++) {
            let VTCFilePath = VTCList[i]
            let VTCFile = bv.GetFileFromFilePath(VTCFilePath)
            let VTCPath = bv.GetPathFromFilePath(VTCFilePath)
            vtcsModel.append({"idx": i+1, "name": VTCFile, "path": VTCPath});
        }
    }
    
    function onSwitchVOIOption()
    {    
        if(pageInputData.subjectSpecificVOIsOp.checked)
        {
            voisModel.clear();
    
            // split subject ID and pure VOI name assuming "<SubjID_VOI>" naming scheme
            var NVOIs = bv.GetPluginIntParameter("NrOfVOIsAvailable");
            var SubjIDs = new Array();
            var PureVOIs = new Array();
            var n_subjects = 0;
            var n_pure_vois = 0;
    
            for(var i=0; i<NVOIs; i++)
            {
                var VOINameID = "NameOfVOI-" + (i+1);
                var VOIName = bv.GetPluginStringParameter(VOINameID);
    
                var s = root.getSubjectID(VOIName); // substring before "_"
                if(s == "") return; // error msg
                var v = root.getPureVOI(VOIName);   // substring after "_"
    
                if(root.arrayContains(SubjIDs, n_subjects, s) == false) // if subj-id not in array, add it
                {
                    //bv.PrintToLog(s);
                    SubjIDs[n_subjects] = s;
                    n_subjects++;
                }
    
                if(root.arrayContains(PureVOIs, n_pure_vois, v) == false) // if pure voi not in array, add it
                {
                    //bv.PrintToLog(v);
                    PureVOIs[n_pure_vois] = v;

                    voisModel.append({"idx": n_pure_vois+1, "name": v})                    

                    n_pure_vois++;
                }
            }            
    
            if(n_pure_vois > 0)
            {
                pageInputData.voisListView.currentIndex = 0;
                // not used, can be deleted; was: from subj-ID's and pure VOI name, we can get original VOI    
                bv.SetPluginStringParameter("SelectedPureVOIName", PureVOIs[0]);
            }
            else
            {
                bv.SetPluginStringParameter("SelectedPureVOIName", "");
    
                // no pure VOIs -> turn off
                pageInputData.subjectSpecificVOIsOp.checked = false;
                root.fillVOIList();
            }
        }
        else
        {
            root.fillVOIList();
        }
    }

    function onGo()
    {
        bv.PrintToLog("Plugin GUI> Dialog's GO button clicked");
    
        var n_vois = voisModel.count;
        if(n_vois == 0) {
            bv.MessageBox("No VOIs available!");
            return;
        }
    
        // get selected VOI:
        var sel_voi = pageInputData.voisListView.currentIndex; // standard or subject-specific VOIs
        if(sel_voi < 0) {
            bv.MessageBox("No VOI selected - please select the VOI serving as reference region for GCM calculation!")
            return;
        }
        bv.SetPluginIntParameter("SelectedVOIIndex", sel_voi); // tell plugin which VOI has been selected - overwritten below if subject-specific VOI
    
        var n_vtcs = vtcsModel.count;
        if(n_vtcs == 0)
        {
            bv.MessageBox("No VTC files available!")
            return;
        }
    
        if(pageEpochSelection.checkBoxSingleIV.checked)
            bv.SetPluginIntParameter("SelectedTimeWindowMethod", 1); // interval
        else
            bv.SetPluginIntParameter("SelectedTimeWindowMethod", 2); // protocol condition
    
        bv.SetPluginStringParameter("SelectedConditionName", pageEpochSelection.condNameTF.text);
    
        var docVMR = bv.ActiveDocument;
    
        // analyze initial substring to find out how many VTCs per subject, then run GCM once for each subject with one or more VTCs
        var SubjIDs = new Array();
        var n_subjects = 0;
        for(var i=0; i<n_vtcs; i++)
        {
            var VTCPathFile = vtcsModel.get(i).name;
            var VTCFile = bv.GetFileFromFilePath(VTCPathFile);
            var s = root.getSubjectID(VTCFile);
            if(s == "") return; // error msg
            if(root.arrayContains(SubjIDs, n_subjects, s) == false) // if subj-id not in array, add it
            {
                //bv.PrintToLog(s);
                SubjIDs[n_subjects] = s;
                n_subjects++;
            }
        }
        bv.PrintToLog("No. of subjects: " + n_subjects);
        bv.SetPluginIntParameter("NrOfSubjects", n_subjects);
    
        if(pageEpochSelection.checkBoxSingleIV.checked)
        {
            bv.SetPluginIntParameter("VTCIntervalStart", pageEpochSelection.intervalFromSpBx.value);
            bv.SetPluginIntParameter("VTCIntervalEnd",   pageEpochSelection.intervalToSpBx.value);
        }
        else
        {
            if(root.checkProtocolsOfAllVTCs() == false)
                return;
    
            bv.SetPluginIntParameter("VTCProtocolCond", pageInputData.vtcsListView.currentIndex);
        }
    
        // in case that subject-specific VOIs selected, check that for each subject there is a VOI defined
        // put all original VOIs in an array to more easier process VOIs below
        var NVOIs = bv.GetPluginIntParameter("NrOfVOIsAvailable");
        var OrigVOIs = new Array();
        var n_orig_vois = 0;
        for(var i=0; i<NVOIs; i++)
        {
            var VOINameID = "NameOfVOI-" + (i+1);
            var VOIName = bv.GetPluginStringParameter(VOINameID);
            OrigVOIs[n_orig_vois] = VOIName;
            n_orig_vois++;
        }
    
        if(pageInputData.subjectSpecificVOIsOp.checked)
        {
            // var SelectedPureVOI = bv.GetPluginStringParameter("SelectedPureVOIName"); // only correct when updated in case user selects a pure VOI in VOIsLW
            var pure_voi_index = pageInputData.voisListView.currentIndex;
            if(pure_voi_index < 0) return;
            var SelectedPureVOI = voisModel.get(pure_voi_index).name;
    
            for(var s=0; s<n_subjects; s++)
            {
                var sName = SubjIDs[s];
    
                var constructed_voi = sName + "_" + SelectedPureVOI;
    
                if(root.arrayContains(OrigVOIs, n_orig_vois, constructed_voi) == false) // if expected VOI does not exist, stop
                {
                    bv.MessageBox("Can not proceed - expected VOI - " + constructed_voi + " - not found!");
                    return;
                }
            }
        }
    
        for(var i=0; i<n_subjects; i++)
        {
            var s = SubjIDs[i];
    
            if(pageInputData.subjectSpecificVOIsOp.checked)
            {
                var constructed_voi = s + "_" + SelectedPureVOI;
                for(var voi_i=0; voi_i<n_orig_vois; voi_i++)
                {
                    if(OrigVOIs[voi_i] == constructed_voi)
                    {
                        bv.SetPluginIntParameter("SelectedVOIIndex", voi_i); // tell plugin which VOI has been selected
    
                        // associate subject with VOI name to simplify access of subject VOI name in plugin code
                        var SubjVOIIndex = "SubjVOIName-" + (i+1);
                        bv.SetPluginStringParameter(SubjVOIIndex, constructed_voi);
    
                        break;
                    }
                }
            }
    
            var n_subj_vtcs = 0;
    
            for(var vtc=0; vtc<n_vtcs; vtc++)
            {
                let VTCPath = vtcsModel.get(vtc).path;
                let VTCFile = vtcsModel.get(vtc).name;
                let VTCPathFile = VTCPath + "/" + VTCFile
    
                if(root.getSubjectID(VTCFile) == s) // VTC belongs to current subject
                {
                    n_subj_vtcs++;
                    var SubjVTCNameID = "SubjVTC-" + n_subj_vtcs;
                    bv.SetPluginStringParameter(SubjVTCNameID, VTCPathFile);
    
                    if(n_subj_vtcs == 1) // if((i==0) && (vtc==0)) // changed: link always correct one to set default folder to appropriate subject folder
                    {
                        docVMR.LinkVTC(VTCPathFile); // since GCM plugin needs (any) VTC, link first one
    
                        // new: we also save for each subject folder (of first VTC) for plugin to allow case that data is not in one folder
                        var SubjFirstVTCFullFileID = "SubjDataFolder-" + (i+1);
                        bv.SetPluginStringParameter(SubjFirstVTCFullFileID, VTCPathFile);
                    }
                }
            }
            bv.PrintToLog(s + " has " + n_subj_vtcs + " subj vtcs");
    
            bv.SetPluginIntParameter("NrOfSubjVTCs", n_subj_vtcs);
    
            // run once for each subject
            bv.SetPluginStringParameter("SubjectID", s);
            bv.SetPluginStringParameter("COMMAND", "RunGCM");
            bv.ExecutePlugin();
        }
    
        // finally, make a single VMP containing all subj maps (one for dGCM, one for Inst)
        for(var i=0; i<n_subjects; i++)
        {
            var s = SubjIDs[i];
            var SubjNameID = "SubjID-" + (i+1);
            bv.SetPluginStringParameter(SubjNameID, s);
        }
        if(pageInputData.subjectSpecificVOIsOp.checked) {
            bv.SetPluginIntParameter("SubjectSpecificVOIs", 1);
        }
        else {
            bv.SetPluginIntParameter("SubjectSpecificVOIs", 0);
        }
    
        bv.SetPluginStringParameter("COMMAND", "PrepareForANCOVA");
        bv.ExecutePlugin();
    
        docVMR.ShowVolumeMapsDlg();
    
        bv.FinishPlugin();
    }
    
    function useProtocolForIntervals()
    {
        condsModel.clear();
    
        var n_vtcs = vtcsModel.count;
        if(n_vtcs == 0) {
            bv.MessageBox("No VTC files available! Add VTC files before switching to this time interval specification method");
            pageEpochSelection.checkBoxSingleIV.checked = true
            pageEpochSelection.checkBoxPRT.checked = false
            return;
        }
    
        // show all conditions of protocol of first VTC for selection (when running RFX-GCM, check that all subsequent protocols have same conds as first VTC, see below)
        let VTCPath = vtcsModel.get(0).path;
        let VTCFile = vtcsModel.get(0).name;
        let VTCPathFile = VTCPath + "/" + VTCFile
        // let plugin provide conditions of VTC's protocol
        bv.SetPluginStringParameter("VTCFile", VTCPathFile);
        bv.SetPluginStringParameter("COMMAND", "GetCondsOfVTCProtocol");
        var ret = bv.ExecutePlugin();
        if(!ret) {
            bv.MessageBox("Error getting conditions from VTCs protocol - protocol attached to all VTCs?");
            return;
        }
    
        var NrOfConditions = bv.GetPluginIntParameter("NrOfConditions");
        for(var c=0; c<NrOfConditions; c++)
        {
            var CondNameID = "CondName-" + (c+1);
            var CondName = bv.GetPluginStringParameter(CondNameID);
            condsModel.append({"idx": c+1, "name": CondName})
        }
        if(NrOfConditions > 1) {
            pageEpochSelection.condsListView.currentIndex = 1; // as default, skip rest (if defined)
            pageEpochSelection.condNameTF.text = condsModel.get(1).name
        }
        else if(NrOfConditions > 0) {
            pageEpochSelection.condsListView.currentIndex = 0;
            pageEpochSelection.condNameTF.text = condsModel.get(0).name
        }
    }
    
    // helper functions
    //
    function getSubjectID(str)
    {
        var idx = str.indexOf("_");
        if(idx >= 0)
            return str.substr(0, idx);
        else
            return "";
    }
    
    function getPureVOI(str)
    {
        var idx = str.indexOf("_");
        if(idx >= 0)
            return str.substr(idx+1, str.length);
        else
            return "";
    }
    
    function arrayContains(arr, n, el)
    {
        for(var i=0; i<n; i++) {
            if(arr[i] == el)
                return true;
        }
        return false;
    }
    
    function checkProtocolsOfAllVTCs()
    {    
        var n_vtcs = vtcsModel.count;
        if(n_vtcs == 0) {
            bv.MessageBox("No VTC files available!");
            return false;
        }
        var n_conds = condsModel.count;
        if(n_conds == 0) {
            bv.MessageBox("No protocol conditions available!");
            return false;
        }
        var sel_cond_idx  = pageEpochSelection.condsListView.currentIndex;
        var sel_cond_name = condsModel.get(sel_cond_idx).name;
    
        // check that all VTCs have a protocol with same condition as selected one for first VTC
        for(var i=0; i<n_vtcs; i++)
        {
            let VTCPath = vtcsModel.get(i).path;
            let VTCFile = vtcsModel.get(i).name;
            let VTCPathFile = VTCPath + "/" + VTCFile
            // let plugin provide conditions of VTC's protocol
            bv.SetPluginStringParameter("VTCFile", VTCPathFile);
            bv.SetPluginStringParameter("COMMAND", "GetCondsOfVTCProtocol");
            var ret = bv.ExecutePlugin();
            if(!ret) {
                bv.MessageBox("Error getting conditions from VTCs protocol - protocol attached to all VTCs?");
                return false;
            }
    
            var NrOfConditions = bv.GetPluginIntParameter("NrOfConditions");
            if(NrOfConditions < n_conds)
                return false;
            var CondNameID = "CondName-" + (sel_cond_idx+1);
            var CondName = bv.GetPluginStringParameter(CondNameID);
            if(CondName != sel_cond_name) {
                bv.MessageBox("Protocol of VTC " + VTCPathFile + " does not have the selected condition, can not determine condition intervals.");
                return false;
            }
        }
        return true;
    }
}
